"""This module take all methods of scripts.__init__.py"""

from mindsight_people_control_api.scripts import (
    AreaRecords,
    Areas,
    EmployeeAreas,
    EmployeeManagers,
    EmployeePositions,
    EmployeeRecord,
    Employees,
    ParentAreas,
    PositionRecords,
    Positions,
    Users,
)

__doc__ = """
**mindsight_people_control_api** is a python package to providing
methods to use People Control rest api.
"""
