"""This module import all methods scripts"""

from mindsight_people_control_api.scripts.area_records import AreaRecords
from mindsight_people_control_api.scripts.areas import Areas
from mindsight_people_control_api.scripts.employee_areas import EmployeeAreas
from mindsight_people_control_api.scripts.employee_managers import EmployeeManagers
from mindsight_people_control_api.scripts.employee_positions import EmployeePositions
from mindsight_people_control_api.scripts.employee_records import EmployeeRecord
from mindsight_people_control_api.scripts.employees import Employees
from mindsight_people_control_api.scripts.parent_areas import ParentAreas
from mindsight_people_control_api.scripts.positions import Positions
from mindsight_people_control_api.scripts.position_records import PositionRecords
from mindsight_people_control_api.scripts.users import Users
